package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.sounds.SoundSource
import net.minecraft.world.Container
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.gameevent.GameEvent
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.capabilities.ForgeCapabilities
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.block.CargoCrateBlock
import ru.dbotthepony.mc.otm.block.IDroppableContainer
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.menu.CargoCrateMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.container.set
import ru.dbotthepony.mc.otm.core.set
import ru.dbotthepony.mc.otm.registry.MSoundEvents

class CargoCrateBlockEntity(
	p_155229_: BlockPos,
	p_155230_: BlockState
) : MatteryBlockEntity(MBlockEntities.CARGO_CRATE, p_155229_, p_155230_), IDroppableContainer {
	val container = MatteryContainer(this::setChanged, CAPACITY)
	private var interactingPlayers = 0
	val handler = container.handler()

	override val droppableContainer: Container
		get() = container

	fun onPlayerOpen() {
		val level = level

		if (interactingPlayers++ == 0 && level != null) {
			level.setBlock(blockPos, blockState.setValue(CargoCrateBlock.IS_OPEN, true), Block.UPDATE_CLIENTS)
			level.playSound(null, blockPos, MSoundEvents.CARGO_CRATE_OPEN, SoundSource.BLOCKS, 1f, 0.8f + level.random.nextFloat() * 0.2f)
			level.gameEvent(GameEvent.CONTAINER_OPEN, blockPos, GameEvent.Context.of(blockState))
		}
	}

	fun onPlayerClose() {
		val level = level

		if (--interactingPlayers == 0 && level != null) {
			level.setBlock(blockPos, blockState.setValue(CargoCrateBlock.IS_OPEN, false), Block.UPDATE_CLIENTS)
			level.gameEvent(GameEvent.CONTAINER_CLOSE, blockPos, GameEvent.Context.of(blockState))
		}
	}

	override fun invalidateCaps() {
		super.invalidateCaps()
		handler.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		handler.revive()
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (cap == ForgeCapabilities.ITEM_HANDLER)
			return handler.get().cast()

		return super.getCapability(cap, side)
	}

	public override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt[INVENTORY_KEY] = container.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		container.deserializeNBT(nbt[INVENTORY_KEY])
	}

	override val defaultDisplayName: Component
		get() = MACHINE_NAME

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return CargoCrateMenu(containerID, inventory, this)
	}

	companion object {
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.cargo_crate")
		const val CAPACITY = 9 * 6
	}
}