package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.world.Container
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.common.capabilities.Capability
import net.minecraftforge.common.capabilities.ForgeCapabilities
import net.minecraftforge.common.util.LazyOptional
import ru.dbotthepony.mc.otm.block.IDroppableContainer
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.capability.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.container.MatteryContainerFilter
import ru.dbotthepony.mc.otm.core.ImpreciseFraction
import ru.dbotthepony.mc.otm.menu.PlatePressMenu
import ru.dbotthepony.mc.otm.registry.MBlockEntities
import ru.dbotthepony.mc.otm.registry.MRecipes
import ru.dbotthepony.mc.otm.container.set
import ru.dbotthepony.mc.otm.core.map
import ru.dbotthepony.mc.otm.core.set

class PlatePressBlockEntity(
	p_155229_: BlockPos,
	p_155230_: BlockState
) : MatteryWorkerBlockEntity<MatteryWorkerBlockEntity.ItemJob>(MBlockEntities.PLATE_PRESS, p_155229_, p_155230_, ::ItemJob), IDroppableContainer {
	val container = MatteryContainer(this::setChangedLight, 2)
	override val energy = WorkerEnergyStorage(this::setChangedLight)

	override val droppableContainer: Container
		get() = container
	val itemHandler = container.handler(object : MatteryContainerFilter {
		override fun canInsert(slot: Int, stack: ItemStack): Boolean {
			return slot != SLOT_OUTPUT
		}

		override fun canExtract(slot: Int, amount: Int, stack: ItemStack): Boolean {
			return slot != SLOT_INPUT
		}
	})

	override fun invalidateCaps() {
		super.invalidateCaps()
		itemHandler.invalidate()
	}

	override fun reviveCaps() {
		super.reviveCaps()
		itemHandler.revive()
	}

	override fun <T> getCapability(cap: Capability<T>, side: Direction?): LazyOptional<T> {
		if (cap == ForgeCapabilities.ITEM_HANDLER)
			return itemHandler.get().cast()

		return super.getCapability(cap, side)
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt[INVENTORY_KEY] = container.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)
		nbt.map(INVENTORY_KEY, container::deserializeNBT)
	}

	override val defaultDisplayName: Component
		get() = MACHINE_NAME

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu {
		return PlatePressMenu(containerID, inventory, this)
	}

	override fun onJobFinish(job: ItemJob): Status {
		if (job.itemStack.isEmpty)
			return Status.SUCCESS

		if (!container.fullyAddItem(job.itemStack, start = SLOT_OUTPUT, end = SLOT_OUTPUT))
			return Status.FAILURE_ITEM

		return Status.SUCCESS
	}

	override fun computeNextJob(): Pair<ItemJob?, IdleReason?> {
		if (energy.batteryLevel.isZero) {
			return null to IdleReason.POWER
		}

		val recipe = level?.recipeManager?.getRecipeFor(MRecipes.PLATE_PRESS, container, level!!)?.orElse(null) ?: return null to IdleReason.ITEM
		container[SLOT_INPUT].shrink(1)
		return ItemJob(recipe.resultItem, recipe.workTime.toDouble(), BASELINE_CONSUMPTION) to null
	}

	companion object {
		private val BASELINE_CONSUMPTION = ImpreciseFraction(15)
		private val MACHINE_NAME = TranslatableComponent("block.overdrive_that_matters.plate_press")
		const val SLOT_INPUT = 0
		const val SLOT_OUTPUT = 1
	}
}
